@@ -11,13 +11,14 @@ import os
 from collections import defaultdict, namedtuple
 from copy import deepcopy
 from subprocess import call
-from typing import List, Tuple  # noqa: F401
+from typing import List, Tuple
 
 import pytest
 from unittest import TestCase, main
 
 import univention.admin.modules
 import univention.debug as ud
+import univention.logging
 import univention.testing.strings as uts
 from univention.testing import utils
 from univention.testing.strings import random_string, random_username
@@ -33,8 +34,7 @@ from univention.udm.exceptions import (
 )
 
 
-ud.init('/var/log/univention/directory-manager-cmd.log', ud.FLUSH, 0)
-ud.set_level(ud.ADMIN, ud.ALL)
+univention.logging.basicConfig(filename='/var/log/univention/directory-manager-cmd.log', univention_debug_level=ud.ALL)
 
 PostalAddress = namedtuple('PostalAddress', ['street', 'zipcode', 'city'])
 
@@ -659,7 +659,7 @@ class TestEncoders(TestCase):
 
 
 @pytest.fixture()
-def simple_udm(ucr):  # type: () -> UDM
+def simple_udm(ucr) -> UDM:
     account = utils.UCSTestDomainAdminCredentials()
     return UDM.credentials(
         account.binddn,
@@ -672,9 +672,9 @@ def simple_udm(ucr):  # type: () -> UDM
 
 @pytest.fixture()
 def schedule_delete_udm_obj(simple_udm):
-    objs = []  # type: List[Tuple[str, str]]
+    objs: List[Tuple[str, str]] = []
 
-    def _func(dn, udm_mod):  # type: (str, str) -> None
+    def _func(dn: str, udm_mod: str) -> None:
         objs.append((dn, udm_mod))
 
     yield _func
