@@ -1,10 +1,14 @@
 #!/usr/bin/python3
+
+from __future__ import annotations
+
 import datetime
 import importlib
 import logging
 import os
 import os.path
 import shutil
+import signal
 import subprocess
 import sys
 import tempfile
@@ -12,6 +16,8 @@ import time
 from contextlib import contextmanager
 from http.client import HTTPConnection
 from shlex import quote
+from types import TracebackType
+from typing import Callable, Dict, Iterator, List, Sequence, Type
 from urllib.parse import urlparse
 
 import requests
@@ -24,7 +30,7 @@ logger = logging.getLogger(__name__)
 PROVIDER_PORTAL_JSON = 'https://provider-portal.software-univention.de/appcenter-selfservice/univention-appcenter-catalog.json'
 
 
-def run_test_file(fname):
+def run_test_file(fname: str) -> None:
     with tempfile.NamedTemporaryFile(suffix='.py') as tmpfile:
         logger.info(f'Copying file to {tmpfile.name}')
         shutil.copy2(fname, tmpfile.name)
@@ -39,10 +45,10 @@ def run_test_file(fname):
 
 
 @contextmanager
-def pip_modules(modules):
+def pip_modules(modules: List[str]) -> Iterator[None]:
     if os.environ.get('UCS_TEST_NO_PIP') == 'TRUE':
         yield
-    if subprocess.run(['which', 'pip3'], stdout=subprocess.DEVNULL).returncode != 0:  # noqa: PLW1510
+    if not shutil.which('pip3'):
         raise RuntimeError('pip3 is required. Install python3-pip')
     installed = subprocess.run(['pip3', 'list', '--format=columns'], stdout=subprocess.PIPE)  # noqa: PLW1510
     logger.info(modules)
@@ -66,9 +72,9 @@ def pip_modules(modules):
 
 
 @contextmanager
-def xserver():
-    if os.environ.get('DISPLAY'):
-        display = os.environ['DISPLAY'].split(':')[1]
+def xserver() -> Iterator[str]:
+    _host, _, display = os.environ.get('DISPLAY', "").partition(":")
+    if display:
         yield display
     else:
         from xvfbwrapper import Xvfb
@@ -76,21 +82,21 @@ def xserver():
             yield xvfb.new_display
 
 
-def ffmpg_start(capture_video, display):
+def ffmpg_start(capture_video: str, display: str) -> int:
     process = subprocess.Popen(['ffmpeg', '-y', '-video_size', '1920x1080', '-framerate', '30', '-f', 'x11grab', '-i', f':{display}', '-c:v', 'libx264', '-crf', '0', capture_video], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
     return process.pid
 
 
-def ffmpg_stop(pid):
-    subprocess.Popen(['kill', str(pid)])
+def ffmpg_stop(pid: int) -> None:
+    os.kill(pid, signal.SIGTERM)
 
 
-def is_local():
+def is_local() -> bool:
     return os.path.exists('/usr/sbin/univention-upgrade')
 
 
 class Session:
-    def __init__(self, display_num, base_url, screenshot_path, driver):
+    def __init__(self, display_num: str, base_url: str, screenshot_path: str, driver) -> None:
         self.display_num = display_num
         self.base_url = base_url
         self.screenshot_path = screenshot_path
@@ -99,8 +105,7 @@ class Session:
         if os.path.isfile(self.ucs_root_ca) and os.environ.get('UCS_TEST_APPS_ADD_CERT', 'yes') != 'no':
             self.add_ucs_root_ca_to_chrome_cert_store()
 
-    def add_ucs_root_ca_to_chrome_cert_store(self):
-        # add ucs root ca to chromiums cert store
+    def add_ucs_root_ca_to_chrome_cert_store(self) -> None:
         # certutil -L -d sql:.pki/nssdb/
         # certutil -A -n "UCS root CA" -t "TCu,Cu,Tu" -i /etc/univention/ssl/ucsCA/CAcert.pem -d sql:.pki/nssd
         cert_store = os.path.join(os.environ['HOME'], '.pki', 'nssdb')
@@ -109,17 +114,17 @@ class Session:
         import_cert = ['certutil', '-A', '-n', 'UCS root CA', '-t', 'TCu,Cu,Tu', '-i', self.ucs_root_ca, '-d', f'sql:{cert_store}']
         subprocess.check_output(import_cert)
 
-    def __enter__(self):
-        yield self
+    def __enter__(self):  # -> Session # Py3.9: Self
+        yield self  # FIXME?
 
-    def __exit__(self, exc_type, exc_value, traceback):
+    def __exit__(self, exc_type: Type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None) -> None:
         self.driver.quit()
 
-    def __del__(self):
+    def __del__(self) -> None:
         self.driver.quit()
 
     @contextmanager
-    def capture(self, name):
+    def capture(self, name: str) -> Iterator[None]:
         filename = self._new_filename(name, 'mkv')
         pid = ffmpg_start(filename, self.display_num)
         try:
@@ -128,23 +133,23 @@ class Session:
             ffmpg_stop(pid)
             self.save_screenshot(name)
 
-    def wait_until_clickable(self, css):
+    def wait_until_clickable(self, css: str) -> None:
         from selenium.webdriver.common.by import By
         from selenium.webdriver.support import expected_conditions as EC
         from selenium.webdriver.support.ui import WebDriverWait
         WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable((By.CSS_SELECTOR, css)))
 
-    def wait_until_clickable_and_click(self, css):
+    def wait_until_clickable_and_click(self, css: str) -> None:
         self.wait_until_clickable(css)
         self.click_element(css)
 
-    def wait_until_gone(self, css):
+    def wait_until_gone(self, css: str) -> None:
         from selenium.webdriver.common.by import By
         from selenium.webdriver.support import expected_conditions as EC
         from selenium.webdriver.support.ui import WebDriverWait
         WebDriverWait(self.driver, 10).until(EC.invisibility_of_element_located((By.CSS_SELECTOR, css)))
 
-    def goto_portal(self):
+    def goto_portal(self) -> None:
         self.get('/univention/portal')
         time.sleep(3)
         if self.find_first('span.umcApplianceReadmeCloseButton'):
@@ -173,14 +178,14 @@ class Session:
         # close side menu
         self.wait_until_clickable_and_click('#header-button-menu')
 
-    def portal_login(self, username, password):
+    def portal_login(self, username: str, password: str) -> None:
         self.wait_until_clickable_and_click('#header-button-menu')
         self.wait_until_clickable_and_click('#loginButton')
         self.enter_input('username', username)
         self.enter_input('password', password)
         self.enter_return()
 
-    def click_portal_tile(self, name):
+    def click_portal_tile(self, name: str) -> None:
         from selenium.common.exceptions import NoSuchElementException
         elements = self.find_all('.portal-tile__name')
         for element in elements:
@@ -199,38 +204,38 @@ class Session:
             raise RuntimeError(f'Could not find {name}')
 
     @contextmanager
-    def switched_frame(self, css):
+    def switched_frame(self, css: str) -> Iterator[None]:
         iframe = self.assert_one(css)
         self.driver.switch_to.frame(iframe)
         yield
         self.driver.switch_to.default_content()
 
-    def get(self, url):
+    def get(self, url: str) -> None:
         if url.startswith('/'):
             url = self.base_url + url
         self.driver.get(url)
 
-    def get_current_url(self):
+    def get_current_url(self) -> str:
         return self.driver.current_url
 
-    def reload(self):
+    def reload(self) -> None:
         self.driver.refresh()
 
-    def find_all(self, css):
+    def find_all(self, css: str) -> List:
         logger.info("Searching for %r", css)
         return self.driver.find_elements(By.CSS_SELECTOR, css)
 
-    def find_all_below(self, element, css):
+    def find_all_below(self, element, css: str) -> List:
         return element.find_elements(By.CSS_SELECTOR, css)
 
-    def find_first(self, css):
+    def find_first(self, css: str):
         elements = self.find_all(css)
         logger.info("Found %d elements", len(elements))
         if len(elements) == 0:
             return None
         return elements[0]
 
-    def assert_one(self, css):
+    def assert_one(self, css: str):
         elements = self.find_all(css)
         assert len(elements) == 1, f'len(elements) == {len(elements)}'
         return elements[0]
@@ -240,22 +245,22 @@ class Session:
         assert len(elements) == 1, f'len(elements) == {len(elements)}'
         return elements[0]
 
-    def click_element(self, css):
+    def click_element(self, css: str) -> None:
         self.assert_one(css).click()
 
-    def click_element_below(self, element, css):
+    def click_element_below(self, element, css: str) -> None:
         self.assert_one_below(element, css).click()
 
-    def change_tab(self, idx):
+    def change_tab(self, idx: int) -> None:
         self.driver.switch_to.window(self.driver.window_handles[idx])
 
-    def close_tab(self):
+    def close_tab(self) -> None:
         self.driver.close()
 
-    def enter_input(self, input_name, value):
+    def enter_input(self, input_name: str, value: str) -> None:
         self.enter_input_element(f'[name={input_name}]', value)
 
-    def enter_input_element(self, css, value):
+    def enter_input_element(self, css: str, value: str) -> None:
         from selenium.common.exceptions import InvalidElementStateException
         elem = self.assert_one(css)
         try:
@@ -264,47 +269,47 @@ class Session:
             pass
         elem.send_keys(value)
 
-    def enter_return(self, css=None):
+    def enter_return(self, css: str | None = None) -> None:
         from selenium.webdriver.common.keys import Keys
         if css:
             self.enter_input_element(css, Keys.RETURN)
         else:
             self.send_keys(Keys.RETURN)
 
-    def enter_shift_tab(self):
+    def enter_shift_tab(self) -> None:
         from selenium.webdriver import ActionChains
         from selenium.webdriver.common.keys import Keys
         action = ActionChains(self.driver)
         action.key_down(Keys.SHIFT).send_keys(Keys.TAB).key_up(Keys.SHIFT)
         action.perform()
 
-    def enter_tab(self):
+    def enter_tab(self) -> None:
         from selenium.webdriver.common.keys import Keys
         self.send_keys(Keys.TAB)
 
-    def drag_and_drop(self, source, target):
+    def drag_and_drop(self, source, target) -> None:
         from selenium.webdriver import ActionChains
         action = ActionChains(self.driver)
         action.drag_and_drop(source, target)
 
-    def send_keys(self, keys):
+    def send_keys(self, keys) -> None:
         from selenium.webdriver import ActionChains
         action = ActionChains(self.driver)
         action.send_keys(keys).perform()
 
-    def _new_filename(self, name, ext):
+    def _new_filename(self, name: str, ext: str) -> str:
         timestamp = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
         os.makedirs(self.screenshot_path, exist_ok=True)
         return os.path.join(self.screenshot_path, f'{name}_{timestamp}.{ext}')
 
-    def save_screenshot(self, name):
+    def save_screenshot(self, name: str) -> str:
         filename = self._new_filename(name, 'png')
         logger.info('Saving screenshot %r', filename)
         self.driver.save_screenshot(filename)
         return filename
 
     @classmethod
-    def chrome(cls, display_num, base_url, screenshot_path):
+    def chrome(cls, display_num: str, base_url: str, screenshot_path: str) -> Session:
         from selenium import webdriver
         options = webdriver.ChromeOptions()
         options.add_argument('--no-sandbox')  # chrome complains about being executed as root
@@ -315,7 +320,7 @@ class Session:
 
     @classmethod
     @contextmanager
-    def running_chrome(cls, base_url, screenshot_path):
+    def running_chrome(cls, base_url: str, screenshot_path: str) -> Iterator[Session]:
         with xserver() as display:
             obj = cls.chrome(display, base_url, screenshot_path)
             obj.driver.maximize_window()
@@ -329,22 +334,20 @@ except ImportError:
     pass
 else:
     @pytest.fixture(scope='session')
-    def config():
+    def config() -> Dict[str, str]:
         """
         Test wide Configuration aka UCR
         Used to get some defaults if not environment variables are
         given. But if UCR is not avaiable, returns an empty dict...
         """
         try:
-            from univention.config_registry import ConfigRegistry
-            ucr = ConfigRegistry()
-            ucr.load()
+            from univention.config_registry import ucr
             return dict(ucr)
         except ImportError:
             return {}
 
     @pytest.fixture(scope='session')
-    def hostname(config):
+    def hostname(config) -> str | None:
         """Hostname to test against"""
         ret = os.environ.get('UCS_TEST_HOSTNAME')
         if ret is None:
@@ -356,7 +359,7 @@ else:
         return ret
 
     @pytest.fixture(scope='session')
-    def fqdn(config):
+    def fqdn(config) -> str:
         """fqdn to test against"""
         ret = os.environ.get('UCS_TEST_HOSTNAME')
         if ret is None:
@@ -370,7 +373,7 @@ else:
         return ret
 
     @pytest.fixture(scope='session')
-    def admin_username(config):
+    def admin_username(config) -> str:
         """Username of the Admin account"""
         ret = os.environ.get('UCS_TEST_ADMIN_USERNAME')
         if not ret:
@@ -379,31 +382,30 @@ else:
         return ret
 
     @pytest.fixture(scope='session')
-    def admin_password(config):
+    def admin_password(config) -> str:
         """Password of the Admin account"""
-        ret = os.environ.get('UCS_TEST_ADMIN_USERNAME')
         ret = os.environ.get('UCS_TEST_ADMIN_PASSWORD')
         if not ret:
-            ret = config.get('tests/domainadmin/pwd', 'univention')
+            ret = config.get('tests/domainadmin/pwd') or 'univention'
         return ret
 
     @pytest.fixture(scope='session')
-    def umc(hostname, admin_username, admin_password):
-        umc_lib = os.environ.get('UCS_TEST_UMC_CLIENT_LIB', 'univention.testing._umc')
+    def umc(hostname: str, admin_username: str, admin_password: str):
+        lib_name = os.environ.get('UCS_TEST_UMC_CLIENT_LIB', 'univention.testing._umc')
         try:
-            umc_lib = importlib.import_module(umc_lib)
+            umc_lib = importlib.import_module(lib_name)
         except ImportError:
             logger.critical(f'Could not import {umc_lib}. Maybe set $UCS_TEST_UMC_CLIENT_LIB')
             raise
         Client = umc_lib.Client
-        scheme, hostname = hostname.split('//')
-        if scheme == 'http:':
+        scheme, _, hostname = hostname.partition("://")
+        if scheme == 'http':
             Client.ConnectionType = HTTPConnection
         client = Client(hostname=hostname, username=admin_username, password=admin_password, useragent='UCS/ucs-test')
         return client
 
     @pytest.fixture()
-    def ucs_call(fqdn):
+    def ucs_call(fqdn: str) -> Callable[[Sequence[str]], None]:
         def _run(args):
             logger.info(f'Running: {args!r}')
             if is_local():
@@ -417,18 +419,18 @@ else:
     @pytest.fixture()
     def ucr(umc):
         class UCR:
-            def __init__(self, client):
+            def __init__(self, client) -> None:
                 self.client = client
-                self._old = {}
+                self._old: Dict[str, str] = {}
 
-            def get(self, key):
+            def get(self, key: str):
                 logger.info('Getting UCRV %s', key)
                 response = self.client.umc_command('ucr/get', [key])
                 value = response.result[0]['value']
                 logger.info('Found %r', value)
                 return value
 
-            def set(self, updates, revert_afterwards=True):
+            def set(self, updates: dict, revert_afterwards=True) -> None:
                 if revert_afterwards:
                     keys = list(updates.keys())
                     response = self.client.umc_command('ucr/get', keys)
@@ -455,10 +457,10 @@ else:
     @pytest.fixture(scope='session')
     def appcenter(umc, fqdn):
         class AppCenter:
-            def __init__(self, client):
+            def __init__(self, client) -> None:
                 self.client = client
 
-            def install_newest(self, app_id):
+            def install_newest(self, app_id: str):
                 app_version = None
                 if '=' in app_id:
                     app_id, app_version = tuple(app_id.split('=', 1))
@@ -505,7 +507,7 @@ else:
                 assert app['is_installed']
                 return app
 
-            def get_published_version(self, app_id):
+            def get_published_version(self, app_id: str):
                 logger.info(f'Retrieving published App {app_id}')
                 r = requests.get(PROVIDER_PORTAL_JSON)
                 for app in r.json():
@@ -513,7 +515,7 @@ else:
                         return app['version']
                 return None
 
-            def get(self, app_id):
+            def get(self, app_id: str):
                 logger.info(f'Retrieving App {app_id}')
                 response = self.client.umc_command('appcenter/get', {'application': app_id})
                 return response.result
@@ -521,11 +523,11 @@ else:
         return AppCenter(umc)
 
     @pytest.fixture(scope='session')
-    def udm(hostname, config, admin_username, admin_password):
+    def udm(hostname: str, config, admin_username: str, admin_password: str):
         """A UDM instance (REST client)"""
-        rest_lib = os.environ.get('UCS_TEST_REST_CLIENT_LIB', 'univention.testing._udm_rest')
+        lib_name = os.environ.get('UCS_TEST_REST_CLIENT_LIB', 'univention.admin.rest.client')
         try:
-            rest_lib = importlib.import_module(rest_lib)
+            rest_lib = importlib.import_module(lib_name)
         except ImportError:
             logger.critical(f'Could not import {rest_lib}. Maybe set $UCS_TEST_REST_CLIENT_LIB')
             raise
@@ -540,12 +542,12 @@ else:
         return udm
 
     @pytest.fixture(scope='session')
-    def users(udm):
+    def users(udm) -> Iterator:
         user_mod = udm.get('users/user')
         users = {}
         user_id_cache = {'X': 1}  # very elegant...
 
-        def _users(user_id=None, attrs={}):
+        def _users(user_id: int | None = None, attrs={}):
             # create mail domain
             if 'mailPrimaryAddress' in attrs:
                 my_domain = attrs['mailPrimaryAddress'].split('@', 1)[1]
@@ -585,10 +587,12 @@ else:
             user = users[username]
             user.reload()
             return user
+
         try:
             yield _users
         finally:
             for user in users.values():
+                user.etag = user.last_modified = None
                 user.delete()
 
     @pytest.fixture()
@@ -610,7 +614,7 @@ else:
             yield conn
 
     @pytest.fixture(scope='session')
-    def selenium_base_url(hostname):
+    def selenium_base_url(hostname: str) -> str:
         """Base URL for selenium"""
         ret = os.environ.get('UCS_TEST_SELENIUM_BASE_URL')
         if ret is None:
@@ -620,7 +624,7 @@ else:
         return ret
 
     @pytest.fixture(scope='session')
-    def selenium_screenshot_path():
+    def selenium_screenshot_path() -> str:
         """Path where selenium should save screenshots"""
         ret = os.environ.get('UCS_TEST_SELENIUM_SCREENSHOT_PATH')
         if ret is None:
@@ -630,12 +634,12 @@ else:
         return os.path.abspath(ret)
 
     @pytest.fixture()
-    def test_logger():
+    def test_logger() -> logging.Logger:
         """Our logger instance so you can print some info for pytest"""
         return logger
 
     @pytest.fixture()
-    def chrome(selenium_base_url, selenium_screenshot_path):
+    def chrome(selenium_base_url: str, selenium_screenshot_path: str) -> Iterator:
         """A running chrome instance, controllable by selenium"""
         with Session.running_chrome(selenium_base_url, selenium_screenshot_path) as c:
             yield c
