@@ -5,6 +5,8 @@ This program compares LDAP host entries with a local comparative ldif file.
 All differences will be displayed at the console.
 """
 
+from __future__ import annotations
+
 import base64
 import errno
 import os
@@ -15,10 +17,10 @@ import subprocess
 import sys
 import time
 import unicodedata
-from optparse import SUPPRESS_HELP, OptionGroup, OptionParser, Values  # noqa: F401
-from typing import Any, Dict, Iterable, Iterator, List, NoReturn, Set, Tuple  # noqa: F401
+from optparse import SUPPRESS_HELP, OptionGroup, OptionParser, Values
+from typing import Any, Dict, Iterable, Iterator, List, NoReturn, Set, Tuple
 
-from typing_extensions import Literal  # noqa: F401
+from typing_extensions import Literal
 
 
 Entry = Dict[str, List[str]]
@@ -71,14 +73,12 @@ class Ldif:
         'structuralObjectClass',
     }
 
-    def __init__(self, src, exclude=OPERATIONAL):
-        # type: (Iterable[bytes], Set[str]) -> None
+    def __init__(self, src: Iterable[bytes], exclude: Set[str] = OPERATIONAL) -> None:
         self.src = src
         self.exclude = exclude
         self.lno = 0
 
-    def next_line(self):
-        # type: () -> Iterator[str]
+    def next_line(self) -> Iterator[str]:
         """Return line iterator."""
         lines = []
         for lno, chunk in enumerate(self.src, start=1):
@@ -93,8 +93,7 @@ class Ldif:
 
         yield ''.join(lines)
 
-    def split(self, line):
-        # type: (str) -> Tuple[str, str]
+    def split(self, line: str) -> Tuple[str, str]:
         r"""
         Split attribute and value.
         Options are stripped.
@@ -130,10 +129,9 @@ class Ldif:
             value = ""
         return (key, value)
 
-    def __iter__(self):
-        # type: () -> Iterator[Entry]
+    def __iter__(self) -> Iterator[Entry]:
         """Return line iterator."""
-        obj = {}   # type: Entry
+        obj: Entry = {}
         for line in self.next_line():
             if line.startswith('#'):
                 continue
@@ -147,8 +145,7 @@ class Ldif:
                 obj = {}
 
     @staticmethod
-    def printable(value):
-        # type: (str) -> str
+    def printable(value: str) -> str:
         """Convert binary data to printable string."""
         # Py2 has no str.isprintable()
         return ''.join(
@@ -159,12 +156,10 @@ class Ldif:
 
 class LdifSource:
     @classmethod
-    def create(cls, arg, options):
-        # type: (str, Values) -> LdifFile
+    def create(cls, arg: str, options: Values) -> LdifFile:
         raise NotImplementedError()
 
-    def start_reading(self):
-        # type: () -> Ldif
+    def start_reading(self) -> Ldif:
         """Start reading the LDIF data."""
         raise NotImplementedError()
 
@@ -173,17 +168,14 @@ class LdifFile:
     """LDIF source from local file."""
 
     @classmethod
-    def create(cls, arg, options):
-        # type: (str, Values) -> LdifFile
+    def create(cls, arg: str, options: Values) -> LdifFile:
         return cls(arg)
 
-    def __init__(self, filename):
-        # type: (str) -> None
+    def __init__(self, filename: str) -> None:
         super().__init__()
         self.filename = filename
 
-    def start_reading(self):
-        # type: () -> Ldif
+    def start_reading(self) -> Ldif:
         """Start reading the LDIF data."""
         try:
             return Ldif(open(self.filename, 'rb'))
@@ -195,17 +187,14 @@ class LdifSlapcat:
     """LDIF source from local LDAP."""
 
     @classmethod
-    def create(cls, arg, options):
-        # type: (Any, Values) -> LdifSlapcat
+    def create(cls, arg: Any, options: Values) -> LdifSlapcat:
         return cls()
 
-    def __init__(self):
-        # type: () -> None
+    def __init__(self) -> None:
         super().__init__()
         self.command = ['slapcat', '-f', '/etc/ldap/slapd.conf', '-d0']
 
-    def start_reading(self):
-        # type: () -> Ldif
+    def start_reading(self) -> Ldif:
         """Start reading the LDIF data."""
         try:
             proc = subprocess.Popen(self.command, stdout=subprocess.PIPE)
@@ -215,8 +204,7 @@ class LdifSlapcat:
         except OSError as ex:
             raise SlapError("Error executing", self.command, ex)
 
-    def wait_for_data(self, proc):
-        # type: (subprocess.Popen) -> None
+    def wait_for_data(self, proc: subprocess.Popen) -> None:
         """
         Wait for the remote process to send data.
 
@@ -228,8 +216,8 @@ class LdifSlapcat:
         """
         while True:
             rlist = [proc.stdout]
-            wlist = []  # type: List[int]
-            xlist = []  # type: List[int]
+            wlist: List[int] = []
+            xlist: List[int] = []
             try:
                 rlist, wlist, xlist = select.select(rlist, wlist, xlist)
                 break
@@ -248,26 +236,22 @@ class LdifSsh(LdifSlapcat):
     """LDIF source from remote LDAP."""
 
     @classmethod
-    def create(cls, hostname, options):
-        # type: (str, Values) -> LdifSsh
+    def create(cls, hostname: str, options: Values) -> LdifSsh:
         return cls(hostname, options.ssh)
 
-    def __init__(self, hostname, ssh='ssh'):
-        # type: (str, str) -> None
+    def __init__(self, hostname: str, ssh: str = 'ssh') -> None:
         super().__init__()
         self.command = [ssh, hostname] + self.command
 
 
-def __test(_option, _opt_str, _value, _parser):
-    # type: (Values, str, None, OptionParser) -> NoReturn
+def __test(_option: Values, _opt_str: str, _value: None, _parser: OptionParser) -> NoReturn:
     """Run internal test suite."""
     import doctest
     res = doctest.testmod()
     sys.exit(int(bool(res[0])))
 
 
-def stream2object(ldif):
-    # type: (Ldif) -> Dict[str, Entry]
+def stream2object(ldif: Ldif) -> Dict[str, Entry]:
     """
     Convert LDIF stream to dictionary of objects.
 
@@ -277,7 +261,7 @@ def stream2object(ldif):
     >>> stream2object([{'dn': ['dc=test']}])
     {'dc=test': {}}
     """
-    objects = {}  # type: Dict[str, Entry]
+    objects: Dict[str, Entry] = {}
     for obj in ldif:
         try:
             dname, = obj.pop('dn')
@@ -289,8 +273,7 @@ def stream2object(ldif):
     return objects
 
 
-def sort_dn(dname):
-    # type: (str) -> Tuple[Tuple[str, ...], ...]
+def sort_dn(dname: str) -> Tuple[Tuple[str, ...], ...]:
     """
     Sort by reversed dn.
 
@@ -307,8 +290,7 @@ def sort_dn(dname):
     return tuple(reversed([tuple(sorted(_.split('+'))) for _ in dname.split(',')]))
 
 
-def compare_ldif(lldif, rldif, options):
-    # type: (Ldif, Ldif, Values) -> int
+def compare_ldif(lldif: Ldif, rldif: Ldif, options: Values) -> int:
     """
     Compare two LDIF files.
 
@@ -356,8 +338,7 @@ def compare_ldif(lldif, rldif, options):
     return ret
 
 
-def compare_keys(ldata, rdata):
-    # type: (Entry, Entry) -> Iterator[Tuple[Literal[-1, 0, 1], str, str]]
+def compare_keys(ldata: Entry, rdata: Entry) -> Iterator[Tuple[Literal[-1, 0, 1], str, str]]:
     """
     Compare and return attributes of two LDAP objects.
 
@@ -399,8 +380,7 @@ def compare_keys(ldata, rdata):
             lkey = rkey = ""
 
 
-def compare_values(attr, lvalues, rvalues):
-    # type: (str, List[str], List[str]) -> Iterator[Tuple[Literal[-1, 0, 1], str, str]]
+def compare_values(attr: str, lvalues: List[str], rvalues: List[str]) -> Iterator[Tuple[Literal[-1, 0, 1], str, str]]:
     """
     Compare and return values of two multi-valued LDAP attributes.
 
@@ -436,8 +416,7 @@ def compare_values(attr, lvalues, rvalues):
             lval = rval = ""
 
 
-def parse_args():
-    # type: () -> Tuple[LdifSource, LdifSource, Values]
+def parse_args() -> Tuple[LdifSource, LdifSource, Values]:
     """Parse command line arguments."""
     parser = OptionParser(usage=USAGE, description=DESCRIPTION)
     parser.disable_interspersed_args()
@@ -500,8 +479,7 @@ def parse_args():
     return ldif1, ldif2, options
 
 
-def main():
-    # type: () -> None
+def main() -> None:
     """A main()-method with options."""
     src1, src2, options = parse_args()
 
@@ -518,8 +496,7 @@ def main():
     run_compare(ldif1, ldif2, options)
 
 
-def run_compare(ldif1, ldif2, options):
-    # type: (Ldif, Ldif, Values) -> NoReturn
+def run_compare(ldif1: Ldif, ldif2: Ldif, options: Values) -> NoReturn:
     """
     UNIX correct error handling.
     Termination by signal is propagaed as signal.
